@@ -826,8 +826,8 @@ librdf_hash_get_all_iterator_next_method(void* iterator)
 static void*
 librdf_hash_get_all_iterator_get_method(void* iterator, int flags) 
 {
-  librdf_hash_get_all_iterator_context* context=(librdf_hash_get_all_iterator_context*)iterator;
-  void *result=NULL;
+  librdf_hash_get_all_iterator_context* context = (librdf_hash_get_all_iterator_context*)iterator;
+  void *result = NULL;
   
   if(context->is_end)
     return NULL;
@@ -839,22 +839,22 @@ librdf_hash_get_all_iterator_get_method(void* iterator, int flags)
        * might as well return something useful to signify not-end-of-list.
        */
 
-      result=&context;
+      result = context;
       break;
 
     case LIBRDF_ITERATOR_GET_METHOD_GET_KEY:
-      result=&context->next_key;
+      result = &context->next_key;
       break;
       
     case LIBRDF_ITERATOR_GET_METHOD_GET_VALUE:
-      result=&context->next_value;
+      result = &context->next_value;
       break;
 
     default:
       librdf_log(context->hash->world, 
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_HASH, NULL,
                  "Unknown iterator method flag %d", flags);
-      result=NULL;
+      result = NULL;
       break;
   }
 
@@ -1086,9 +1086,9 @@ librdf_hash_keys_iterator_next_method(void* iterator)
 static void*
 librdf_hash_keys_iterator_get_method(void* iterator, int flags) 
 {
-  librdf_hash_keys_iterator_context* context=(librdf_hash_keys_iterator_context*)iterator;
-  void *result=NULL;
-  
+  librdf_hash_keys_iterator_context* context = (librdf_hash_keys_iterator_context*)iterator;
+  void *result = NULL;
+
   if(context->is_end)
     return NULL;
 
@@ -1099,15 +1099,15 @@ librdf_hash_keys_iterator_get_method(void* iterator, int flags)
        * might as well return something useful to signify not-end-of-list.
        */
 
-      result=&context;
+      result = context;
       break;
       
     case LIBRDF_ITERATOR_GET_METHOD_GET_KEY:
-      result=&context->next_key;
+      result = &context->next_key;
       break;
       
     default:
-      result=NULL;
+      result = NULL;
   }
 
   return result;
@@ -1363,8 +1363,7 @@ librdf_hash_from_string(librdf_hash* hash, const char *string)
 
   p=string;
   key=NULL; key_len=0;
-  value=NULL; value_len=0;
-  backslashes=0;
+  value=NULL;
   state=HFS_PARSE_STATE_INIT;
   while(*p) {
 
@@ -1443,7 +1442,7 @@ librdf_hash_from_string(librdf_hash* hash, const char *string)
           break;
         }
         p++;
-        state=HFS_PARSE_STATE_VALUE;
+        /* state=HFS_PARSE_STATE_VALUE; */
         /* fall through to next state */
         
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
@@ -1660,36 +1659,42 @@ librdf_hash_to_string(librdf_hash* hash, const char *filter[])
  * @key: key string to look up
  *
  * Lookup a hash key and decode value as a boolean.
+ *
+ * False values: "no", "false"
+ * True values: "yes", "true"
  * 
  * Return value: >0 (for true), 0 (for false) or <0 (for key not found or not known boolean value)
  **/
 int
-librdf_hash_get_as_boolean (librdf_hash* hash, const char *key) 
+librdf_hash_get_as_boolean(librdf_hash* hash, const char *key) 
 {
-  int bvalue= (-1);
+  int bvalue = (-1);
   char *value;
 
-  value=librdf_hash_get(hash, key);
+  value = librdf_hash_get(hash, key);
   if(!value)
     /* does not exist - fail */
     return -1;
 
   switch(strlen(value)) {
   case 2: /* try 'no' */
-    if(*value=='n' && value[1]=='o')
-      bvalue=0;
+    if(*value == 'n' && value[1] == 'o')
+      bvalue = 0;
     break;
+
   case 3: /* try 'yes' */
-    if(*value=='y' && value[1]=='e' && value[2]=='s')
-      bvalue=1;
+    if(*value == 'y' && value[1] == 'e' && value[2] == 's')
+      bvalue = 1;
     break;
+
   case 4: /* try 'true' */
-    if(*value=='t' && value[1]=='r' && value[2]=='u' && value[3]=='e')
-      bvalue=1;
+    if(*value == 't' && value[1] == 'r' && value[2] == 'u' && value[3] == 'e')
+      bvalue = 1;
     break;
+
   case 5: /* try 'false' */
     if(!strncmp(value, "false", 5))
-      bvalue=1;
+      bvalue = 0;
     break;
   /* no need for default, bvalue is set above */
   }
