@@ -256,6 +256,9 @@ librdf_get_query_factory(librdf_world *world,
  * 
  * All returned strings are shared and must be copied if needed to be
  * used dynamically.
+ *
+ * @deprecated: use librdf_query_language_get_description() to return
+ * more information in a static structure.
  * 
  * Return value: non 0 on failure of if counter is out of range
  */
@@ -295,6 +298,24 @@ librdf_query_languages_enumerate(librdf_world* world,
   return 1;
 }
 
+/**
+ * librdf_query_language_get_description:
+ * @world: #librdf_world
+ * @counter: index into the list of query language syntaxes
+ *
+ * Get a query language syntax description.
+ *
+ * Return value: description or NULL if counter is out of range
+ */
+const raptor_syntax_description*
+librdf_query_language_get_description(librdf_world* world,
+                                      unsigned int counter)
+{
+  librdf_world_open(world);
+
+  return rasqal_world_get_query_language_description(world->rasqal_world_ptr,
+                                                     counter);
+}
 
 /**
  * librdf_new_query:
@@ -485,15 +506,17 @@ librdf_query_remove_query_result(librdf_query *query,
                                  librdf_query_results* query_results)
 {
   librdf_query_results *cur, *prev=NULL;
-  for(cur=query->results; cur && cur != query_results; cur=cur->next)
-    prev=cur;
+  for(cur = query->results; cur && cur != query_results; cur = cur->next)
+    prev = cur;
   
-  if(cur == query_results) {
-    if(prev)
-      prev->next=cur->next;
+  if(cur != NULL) {
+    if(cur == query_results) {
+      if(prev)
+        prev->next = cur->next;
+    }
+    if(cur == query->results)
+      query->results = cur->next;
   }
-  if(cur == query->results && cur != NULL)
-    query->results=cur->next;
 
   /* remove reference and free if we are the last */
   librdf_free_query(query);
@@ -659,13 +682,15 @@ main(int argc, char *argv[])
   /* list available query languages */
   fprintf(stdout, "%s: Enumerating query languages:\n", program);
   for (i = 0; 1; i++) {
+    const raptor_syntax_description* desc = NULL;
     const char *name;
     const unsigned char *uri;
     
-    if (librdf_query_languages_enumerate(world, i, &name, &uri))
+    desc = librdf_query_language_get_description(world, i);
+    if (!desc)
       break;
       
-    fprintf(stdout, " %s <%s>\n", name, (uri == NULL ? "" : (char*)uri));
+    fprintf(stdout, " %s <%s>\n", desc->names[0], desc->uri_strings_count ? desc->uri_strings[0] : "");
   }
 
   /* create model and storage */
