@@ -1,8 +1,8 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdf_query.c - RDF Query Results
+ * rdf_query.c - RDF Query Results and Query Results Formatter classes
  *
- * Copyright (C) 2004-2008, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2004-2010, David Beckett http://www.dajobe.org/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -36,7 +36,7 @@
 #include <ctype.h>
 #include <sys/types.h>
 #ifdef HAVE_STDLIB_H
-#include <stdlib.h> /* for abort() as used in errors */
+#include <stdlib.h>
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -242,8 +242,9 @@ librdf_query_results_get_bindings_count(librdf_query_results *query_results)
 void
 librdf_free_query_results(librdf_query_results* query_results)
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(query_results, librdf_query_results);
-
+  if(!query_results)
+    return;
+  
   if(query_results->query->factory->free_results)
     query_results->query->factory->free_results(query_results);
 
@@ -254,29 +255,38 @@ librdf_free_query_results(librdf_query_results* query_results)
 
 
 /**
- * librdf_query_results_to_counted_string:
+ * librdf_query_results_to_counted_string2:
  * @query_results: #librdf_query_results object
- * @format_uri: URI of syntax to format to
- * @base_uri: Base URI of output formatted syntax  or NULL
- * @length_p: Pointer to where to store length of string or NULL
+ * @name: name of syntax to format to
+ * @mime_type: mime type of syntax to format to (or NULL)
+ * @format_uri: URI of syntax to format to (or NULL)
+ * @base_uri: Base URI of output formatted syntax (or NULL)
+ * @length_p: Pointer to where to store length of string (or NULL)
  *
  * Turn a query results into a string.
  * 
- * Values of format_uri currently supported (via Rasqal) are:
- *  http://www.w3.org/TR/2004/WD-rdf-sparql-XMLres-20041221/
+ * A query results format can be named, have a mime type, or
+ * identified by a URI, all of which are optional.  The default
+ * query results format will be used if @name, @mime_type and
+ * @format_uri are all NULL.
  *
- * The base URI may be used for the generated syntax, depending
- * on the format.
+ * librdf_query_results_formats_enumerate() returns information on
+ * the known query results names, labels and URIs.
+ *
+ * The @base_uri may be used for as the base URI the generated
+ * syntax, depending on the format.
  *
  * The returned string must be freed by the caller
  *
  * Return value: new string value or NULL on failure
  **/
 unsigned char*
-librdf_query_results_to_counted_string(librdf_query_results *query_results,
-                                       librdf_uri *format_uri,
-                                       librdf_uri *base_uri,
-                                       size_t *length_p)
+librdf_query_results_to_counted_string2(librdf_query_results *query_results,
+                                        const char *name,
+                                        const char *mime_type,
+                                        librdf_uri *format_uri,
+                                        librdf_uri *base_uri,
+                                        size_t *length_p)
 {
   librdf_query_results_formatter *formatter;
   void *string=NULL;
@@ -286,13 +296,14 @@ librdf_query_results_to_counted_string(librdf_query_results *query_results,
 
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_results, librdf_query_results, NULL);
 
-  iostr=raptor_new_iostream_to_string(&string, &string_length, malloc);
+  iostr = raptor_new_iostream_to_string(query_results->query->world->raptor_world_ptr,
+                                        &string, &string_length, malloc);
   if(!iostr)
     return NULL;
               
-  formatter=librdf_new_query_results_formatter(query_results,
-                                               NULL /* name */,
-                                               format_uri);
+  formatter = librdf_new_query_results_formatter2(query_results,
+                                                  name, mime_type,
+                                                  format_uri);
   if(!formatter) {
     error=1;
     goto tidy;
@@ -316,58 +327,167 @@ librdf_query_results_to_counted_string(librdf_query_results *query_results,
     }
   }
   else if(length_p)
-    *length_p=string_length;
+    *length_p = string_length;
   
   return (unsigned char *)string;
 }
 
 
+#ifndef REDLAND_DISABLE_DEPRECATED
 /**
- * librdf_query_results_to_string:
+ * librdf_query_results_to_counted_string:
  * @query_results: #librdf_query_results object
- * @format_uri: URI of syntax to format to
- * @base_uri: Base URI of output formatted syntax 
+ * @format_uri: URI of syntax to format to (or NULL)
+ * @base_uri: Base URI of output formatted syntax (or NULL)
+ * @length_p: Pointer to where to store length of string (or NULL)
  *
  * Turn a query results into a string.
  * 
- * See librdf_query_results_to_counted_string for information on the
- * format_uri and base_uri parameters.
+ * The default query results format will be used if @format_uri is
+ * NULL.
+ *
+ * librdf_query_results_formats_enumerate() returns information on
+ * the known query results names, labels and URIs.
+ *
+ * The @base_uri may be used for as the base URI the generated
+ * syntax, depending on the format.
+ *
+ * The returned string must be freed by the caller
+ *
+ * @deprecated: Use librdf_query_results_to_counted_string2() with extra
+ * name and mime-type args.
+ *
+ * Return value: new string value or NULL on failure
+ **/
+unsigned char*
+librdf_query_results_to_counted_string(librdf_query_results *query_results,
+                                       librdf_uri *format_uri,
+                                       librdf_uri *base_uri,
+                                       size_t *length_p)
+{
+  return librdf_query_results_to_counted_string2(query_results,
+                                                 NULL /* name */,
+                                                 NULL /* mime type */,
+                                                 format_uri,
+                                                 base_uri,
+                                                 length_p);
+}
+#endif
+
+
+/**
+ * librdf_query_results_to_string2:
+ * @query_results: #librdf_query_results object
+ * @name: format name
+ * @mime_type: format mime type (or NULL)
+ * @format_uri: URI of syntax to format to (or NULL)
+ * @base_uri: Base URI of output formatted syntax (or NULL)
+ *
+ * Turn a query results into a string.
+ * 
+ * A query results format can be named, have a mime type, or
+ * identified by a URI, all of which are optional.  The default
+ * query results format will be used if @name, @mime_type and @format_uri
+ * are all NULL.
+ *
+ * librdf_query_results_formats_enumerate() returns information on
+ * the known query results names, labels and URIs.
+ *
+ * The @base_uri may be used for as the base URI the generated
+ * syntax, depending on the format.
  *
  * The returned string must be freed by the caller
  *
  * Return value: new string value or NULL on failure
  **/
 unsigned char*
-librdf_query_results_to_string(librdf_query_results *query_results,
-                               librdf_uri *format_uri,
-                               librdf_uri *base_uri) {
+librdf_query_results_to_string2(librdf_query_results *query_results,
+                                const char* name,
+                                const char* mime_type,
+                                librdf_uri *format_uri,
+                                librdf_uri *base_uri)
+{
 
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_results, librdf_query_results, NULL);
 
-  return librdf_query_results_to_counted_string(query_results, 
-                                                format_uri, base_uri, NULL);
+  return librdf_query_results_to_counted_string2(query_results,
+                                                 name, mime_type, format_uri,
+                                                 base_uri, NULL);
 }
 
 
+#ifndef REDLAND_DISABLE_DEPRECATED
 /**
- * librdf_query_results_to_file_handle:
+ * librdf_query_results_to_string:
+ * @query_results: #librdf_query_results object
+ * @format_uri: URI of syntax to format to
+ * @base_uri: Base URI of output formatted syntax (or NULL)
+ *
+ * Turn a query results into a string.
+ * 
+ * A query results format can be named, have a mime type, or
+ * identified by a URI, all of which are optional.  The default
+ * query results format will be used if @format_uri is NULL.
+ *
+ * librdf_query_results_formats_enumerate() returns information on
+ * the known query results names, labels and URIs.
+ *
+ * The @base_uri may be used for as the base URI the generated
+ * syntax, depending on the format.
+ *
+ * The returned string must be freed by the caller
+ *
+ * @Deprecated: use librdf_query_results_to_string2() with extra name
+ * and mime_type args.
+ *
+ * Return value: new string value or NULL on failure
+ **/
+unsigned char*
+librdf_query_results_to_string(librdf_query_results *query_results,
+                               librdf_uri *format_uri,
+                               librdf_uri *base_uri)
+{
+
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_results, librdf_query_results, NULL);
+
+  return librdf_query_results_to_string2(query_results,
+                                         NULL, NULL, format_uri,
+                                         base_uri);
+}
+#endif
+
+
+/**
+ * librdf_query_results_to_file_handle2:
  * @query_results: #librdf_query_results object
  * @handle: file handle to write to
- * @format_uri: URI of syntax to format to
+ * @name: result format name (or NULL)
+ * @mime_type: result mime type (or NULL)
+ * @format_uri: URI of syntax to format to (or NULL)
  * @base_uri: Base URI of output formatted syntax 
  *
  * Write a query results to a FILE*.
  * 
- * See librdf_query_results_to_counted_string for information on the
- * format_uri and base_uri parameters.
+ * A query results format can be named, have a mime type, or
+ * identified by a URI, all of which are optional.  The default
+ * query results format will be used if @name, @mime_type and @format_uri
+ * are all NULL.
+ *
+ * librdf_query_results_formats_enumerate() returns information on
+ * the known query results names, labels and URIs.
+ *
+ * The @base_uri may be used for as the base URI the generated syntax, depending
+ * on the format.
  *
  * Return value: non 0 on failure
  **/
 int
-librdf_query_results_to_file_handle(librdf_query_results *query_results, 
-                                    FILE *handle, 
-                                    librdf_uri *format_uri,
-                                    librdf_uri *base_uri)
+librdf_query_results_to_file_handle2(librdf_query_results *query_results, 
+                                     FILE *handle, 
+                                     const char *name,
+                                     const char *mime_type,
+                                     librdf_uri *format_uri,
+                                     librdf_uri *base_uri)
 {
   raptor_iostream *iostr;
   librdf_query_results_formatter *formatter;
@@ -377,20 +497,21 @@ librdf_query_results_to_file_handle(librdf_query_results *query_results,
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(handle, FILE*, 1);
 
 
-  iostr=raptor_new_iostream_to_file_handle(handle);
+  iostr = raptor_new_iostream_to_file_handle(query_results->query->world->raptor_world_ptr,
+                                             handle);
   if(!iostr)
     return 1;
-              
-  formatter=librdf_new_query_results_formatter(query_results,
-                                               NULL /* name */,
-                                               format_uri);
+
+  formatter = librdf_new_query_results_formatter2(query_results,
+                                                  name, mime_type,
+                                                  format_uri);
   if(!formatter) {
     raptor_free_iostream(iostr);
     return 1;
   }
 
-  status=librdf_query_results_formatter_write(iostr, formatter,
-                                              query_results, base_uri);
+  status = librdf_query_results_formatter_write(iostr, formatter,
+                                                query_results, base_uri);
 
   librdf_free_query_results_formatter(formatter);
 
@@ -400,32 +521,84 @@ librdf_query_results_to_file_handle(librdf_query_results *query_results,
 }
 
 
+#ifndef REDLAND_DISABLE_DEPRECATED
 /**
- * librdf_query_results_to_file:
+ * librdf_query_results_to_file_handle:
  * @query_results: #librdf_query_results object
- * @name: filename to write to
+ * @handle: file handle to write to
  * @format_uri: URI of syntax to format to
- * @base_uri: Base URI of output formatted syntax 
+ * @base_uri: Base URI of output formatted syntax (or NULL)
  *
- * Write a query results to a file.
+ * Write a query results to a FILE*.
  * 
- * See librdf_query_results_to_counted_string for information on the
- * format_uri and base_uri parameters.
+ * A query results format can be named, have a mime type, or
+ * identified by a URI, all of which are optional.  The default
+ * query results format will be used if @format_uri is NULL.
+ *
+ * librdf_query_results_formats_enumerate() returns information on
+ * the known query results names, labels and URIs.
+ *
+ * The @base_uri may be used for as the base URI the generated
+ * syntax, depending on the format.
+ *
+ * @Deprecated: use librdf_query_results_to_file_handle() with extra
+ * name and mime_type args.
  *
  * Return value: non 0 on failure
  **/
 int
-librdf_query_results_to_file(librdf_query_results *query_results, 
+librdf_query_results_to_file_handle(librdf_query_results *query_results, 
+                                    FILE *handle, 
+                                    librdf_uri *format_uri,
+                                    librdf_uri *base_uri)
+{
+  return librdf_query_results_to_file_handle2(query_results, 
+                                              handle,
+                                              NULL /* name */,
+                                              NULL /* mime type */,
+                                              format_uri,
+                                              base_uri);
+}
+#endif
+
+
+/**
+ * librdf_query_results_to_file2:
+ * @query_results: #librdf_query_results object
+ * @name: filename to write to
+ * @mime_type: mime type (or NULL)
+ * @format_uri: URI of syntax to format to (or NULL)
+ * @base_uri: Base URI of output formatted syntax (or NULL)
+ *
+ * Write a query results to a file.
+ * 
+ * A query results format can be named, have a mime type, or
+ * identified by a URI, all of which are optional.  The default
+ * query results format will be used if @name, @mime_type and @format_uri
+ * are all NULL.
+ *
+ * librdf_query_results_formats_enumerate() returns information on
+ * the known query results names, labels and URIs.
+ *
+ * The @base_uri may be used for as the base URI the generated
+ * syntax, depending on the format.
+ *
+ * Return value: non 0 on failure
+ **/
+int
+librdf_query_results_to_file2(librdf_query_results *query_results, 
                              const char *name,
+                             const char *mime_type,
                              librdf_uri *format_uri,
-                             librdf_uri *base_uri) {
+                             librdf_uri *base_uri)
+{
   FILE* fh;
   int status;
   
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_results, query_results, 1);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(name, string, 1);
 
-  fh=fopen(name, "w+");
+  fh = fopen(name, "w+");
   if(!fh) {
     librdf_log(query_results->query->world, 0, LIBRDF_LOG_ERROR, 
                LIBRDF_FROM_QUERY, NULL, 
@@ -434,13 +607,54 @@ librdf_query_results_to_file(librdf_query_results *query_results,
     return 1;
   }
 
-  status=librdf_query_results_to_file_handle(query_results, fh, 
-                                             format_uri, base_uri);
+  status = librdf_query_results_to_file_handle2(query_results, fh, 
+                                                name, mime_type,
+                                                format_uri, base_uri);
   fclose(fh);
   return status;
 }
 
 
+#ifndef REDLAND_DISABLE_DEPRECATED
+/**
+ * librdf_query_results_to_file:
+ * @query_results: #librdf_query_results object
+ * @name: filename to write to
+ * @format_uri: URI of syntax to format to
+ * @base_uri: Base URI of output formatted syntax (or NULL)
+ *
+ * Write a query results to a file.
+ * 
+ * A query results format can be named, have a mime type, or
+ * identified by a URI, all of which are optional.  The default
+ * query results format will be used if @format_uri is NULL.
+ *
+ * librdf_query_results_formats_enumerate() returns information on
+ * the known query results names, labels and URIs.
+ *
+ * The @base_uri may be used for as the base URI the generated
+ * syntax, depending on the format.
+ *
+ * @Deprecated: use librdf_query_results_to_file2() with extra mime_type
+ * arg.
+ *
+ * Return value: non 0 on failure
+ **/
+int
+librdf_query_results_to_file(librdf_query_results *query_results, 
+                             const char *name,
+                             librdf_uri *format_uri,
+                             librdf_uri *base_uri)
+{
+  return librdf_query_results_to_file2(query_results,
+                                       name,
+                                       NULL /* mime type */,
+                                       format_uri,
+                                       base_uri);
+}
+#endif
+
+
 /**
  * librdf_query_results_is_bindings:
  * @query_results: #librdf_query_results object
@@ -450,7 +664,8 @@ librdf_query_results_to_file(librdf_query_results *query_results,
  * Return value: non-0 if true
  **/
 int
-librdf_query_results_is_bindings(librdf_query_results* query_results) {
+librdf_query_results_is_bindings(librdf_query_results* query_results)
+{
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_results, query_results, -1);
 
   if(query_results->query->factory->results_is_bindings)
@@ -472,7 +687,8 @@ librdf_query_results_is_bindings(librdf_query_results* query_results) {
  * Return value: non-0 if true
  **/
 int
-librdf_query_results_is_boolean(librdf_query_results* query_results) {
+librdf_query_results_is_boolean(librdf_query_results* query_results)
+{
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_results, query_results, -1);
 
   if(query_results->query->factory->results_is_boolean)
@@ -491,7 +707,8 @@ librdf_query_results_is_boolean(librdf_query_results* query_results) {
  * Return value: non-0 if true
  **/
 int
-librdf_query_results_is_graph(librdf_query_results* query_results) {
+librdf_query_results_is_graph(librdf_query_results* query_results)
+{
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_results, query_results, -1);
 
   if(query_results->query->factory->results_is_graph)
@@ -534,12 +751,13 @@ librdf_query_results_is_syntax(librdf_query_results* query_results)
  * Get boolean query result.
  *
  * The return value is only meaningful if this is a boolean
- * query result - see #librdf_query_results_is_boolean
+ * query result - see librdf_query_results_is_boolean()
  *
  * Return value: boolean query result - >0 is true, 0 is false, <0 on error or finished
  */
 int
-librdf_query_results_get_boolean(librdf_query_results* query_results) {
+librdf_query_results_get_boolean(librdf_query_results* query_results)
+{
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_results, query_results, -1);
 
   if(query_results->query->factory->results_get_boolean)
@@ -556,12 +774,13 @@ librdf_query_results_get_boolean(librdf_query_results* query_results) {
  * Get a query result as an RDF graph in #librdf_stream form
  *
  * The return value is only meaningful if this is an RDF graph
- * query result - see #librdf_query_results_is_graph
+ * query result - see librdf_query_results_is_graph().
  *
  * Return value: a new #librdf_stream result or NULL on error
  */
 librdf_stream*
-librdf_query_results_as_stream(librdf_query_results* query_results) {
+librdf_query_results_as_stream(librdf_query_results* query_results)
+{
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_results, query_results, NULL);
 
   if(query_results->query->factory->results_as_stream)
@@ -571,6 +790,38 @@ librdf_query_results_as_stream(librdf_query_results* query_results) {
 }
 
 
+/**
+ * librdf_new_query_results_formatter2:
+ * @query_results: #librdf_query_results query_results
+ * @name: the query results format name (or NULL)
+ * @mime_type: the query results format mime type (or NULL)
+ * @uri: #librdf_uri query results format uri (or NULL)
+ *
+ * Constructor - create a new librdf_query_results_formatter object by identified format.
+ *
+ * A query results format can be named, have a mime type, or
+ * identified by a URI, all of which are optional.  The default query
+ * results format will be used if @name, @mime_type and @uri are all
+ * NULL.
+ *
+ * librdf_query_results_formats_enumerate() returns information on
+ * the known query results names, labels and URIs.
+ *
+ * Return value: a new #librdf_query_results_formatter object or NULL on failure
+ */
+librdf_query_results_formatter*
+librdf_new_query_results_formatter2(librdf_query_results* query_results,
+                                    const char *name, const char *mime_type,
+                                    librdf_uri* uri)
+{
+  if(query_results->query->factory->new_results_formatter)
+    return query_results->query->factory->new_results_formatter(query_results, name, mime_type, uri);
+  else
+    return NULL;
+}
+
+
+#ifndef REDLAND_DISABLE_DEPRECATED
 /**
  * librdf_new_query_results_formatter:
  * @query_results: #librdf_query_results query_results
@@ -581,8 +832,13 @@ librdf_query_results_as_stream(librdf_query_results* query_results) {
  *
  * A query results format can be named or identified by a URI, both
  * of which are optional.  The default query results format will be used
- * if both are NULL.  librdf_query_results_formats_enumerate() returns
- * information on the known query results names, labels and URIs.
+ * if @name and @uri are both NULL.
+ *
+ * librdf_query_results_formats_enumerate() returns information on
+ * the known query results names, labels and URIs.
+ *
+ * @Deprecated: for librdf_new_query_results_formatter2() with the
+ * name, mime_type and format_uri args.
  *
  * Return value: a new #librdf_query_results_formatter object or NULL on failure
  */
@@ -591,12 +847,14 @@ librdf_new_query_results_formatter(librdf_query_results* query_results,
                                    const char *name, librdf_uri* uri)
 {
   if(query_results->query->factory->new_results_formatter)
-    return query_results->query->factory->new_results_formatter(query_results, name, uri);
+    return query_results->query->factory->new_results_formatter(query_results, name, NULL, uri);
   else
     return NULL;
 }
+#endif
 
 
+#ifndef REDLAND_DISABLE_DEPRECATED
 /**
  * librdf_new_query_results_formatter_by_mime_type:
  * @query_results: #librdf_query_results query_results
@@ -612,17 +870,21 @@ librdf_new_query_results_formatter(librdf_query_results* query_results,
  * case the librdf_new_query_results_formatter() constructor allows
  * selecting of a specific one by name or URI.
  *
+ * @Deprecated: for librdf_new_query_results_formatter2() with the
+ * name, mime_type and format_uri args.
+ *
  * Return value: a new #librdf_query_results_formatter object or NULL on failure
  */
 librdf_query_results_formatter*
 librdf_new_query_results_formatter_by_mime_type(librdf_query_results* query_results,
                                                 const char *mime_type)
 {
-  if(query_results->query->factory->new_results_formatter_by_mime_type)
-    return query_results->query->factory->new_results_formatter_by_mime_type(query_results, mime_type);
+  if(query_results->query->factory->new_results_formatter)
+    return query_results->query->factory->new_results_formatter(query_results, NULL, mime_type, NULL);
   else
     return NULL;
 }
+#endif
 
 
 /**
@@ -634,6 +896,9 @@ librdf_new_query_results_formatter_by_mime_type(librdf_query_results* query_resu
 void
 librdf_free_query_results_formatter(librdf_query_results_formatter* formatter) 
 {
+  if(!formatter)
+    return;
+  
   if(formatter->query_results->query->factory->free_results_formatter)
     formatter->query_results->query->factory->free_results_formatter(formatter);
 }
@@ -643,11 +908,14 @@ librdf_free_query_results_formatter(librdf_query_results_formatter* formatter)
  * librdf_query_results_formatter_write:
  * @iostr: #raptor_iostream to write the query to
  * @formatter: #librdf_query_results_formatter object
- * @results: #librdf_query_results query results format
+ * @query_results: #librdf_query_results query results format
  * @base_uri: #librdf_uri base URI of the output format
  *
  * Write the query results using the given formatter to an iostream
  * 
+ * Note that after calling this method, the query results will be
+ * empty and librdf_query_results_finished() will return true (non-0)
+ *
  * See librdf_query_results_formats_enumerate() to get the
  * list of syntax URIs and their description. 
  *
@@ -685,13 +953,16 @@ librdf_query_results_formats_check(librdf_world* world,
                                    const char *name, librdf_uri* uri,
                                    const char *mime_type)
 {
+  int flags = RASQAL_QUERY_RESULTS_FORMAT_FLAG_READER;
+  
   librdf_world_open(world);
 
   /* FIXME - this should use some kind of registration but for now
    * it is safe to assume Rasqal does it all
    */
   return rasqal_query_results_formats_check(world->rasqal_world_ptr,
-                                            name, (raptor_uri*)uri, mime_type);
+                                            name, (raptor_uri*)uri, mime_type,
+                                            flags);
 }
 
 
@@ -709,6 +980,9 @@ librdf_query_results_formats_check(librdf_world* world,
  * All returned strings are shared and must be copied if needed to be
  * used dynamically.
  * 
+ * @Deprecated: use librdf_query_results_formats_get_description() to
+ * return more information in a static structure.
+ *
  * Return value: non 0 on failure of if counter is out of range
  */
 int
@@ -719,14 +993,51 @@ librdf_query_results_formats_enumerate(librdf_world* world,
                                        const unsigned char **uri_string,
                                        const char **mime_type)
 {
+  const raptor_syntax_description *desc;
+
   librdf_world_open(world);
 
   /* FIXME - this should use some kind of registration but for now
    * it is safe to assume Rasqal does it all
    */
-  return rasqal_query_results_formats_enumerate(world->rasqal_world_ptr,
-                                                counter, name, label,
-                                                uri_string, mime_type,
-                                                /*flags=*/ NULL);
+  desc = rasqal_world_get_query_results_format_description(world->rasqal_world_ptr, counter);
+  if(!desc)
+    return -1;
+  
+  /* First element of names, uri_strings, mime_types arrays is always
+   * the main one 
+   */
+  if(name && desc->names)
+    *name = desc->names[0];
+
+  if(label)
+    *label = desc->label;
+
+  if(uri_string && desc->uri_strings)
+    *uri_string = (const unsigned char *)desc->uri_strings[0];
+
+  if(mime_type && desc->mime_types)
+    *mime_type = desc->mime_types[0].mime_type;
+
+  return 0;
 }
 
+
+/**
+ * librdf_query_results_formats_get_description:
+ * @world: world object
+ * @counter: index into the list of query results formats
+ *
+ * Get query result formats descriptive syntax information
+ * 
+ * Return value: description or NULL if counter is out of range
+ **/
+const raptor_syntax_description*
+librdf_query_results_formats_get_description(librdf_world* world, 
+                                             unsigned int counter)
+{
+  librdf_world_open(world);
+
+  return rasqal_world_get_query_results_format_description(world->rasqal_world_ptr,
+                                                           counter);
+}
