@@ -34,7 +34,7 @@
 #include <stdio.h>
 #include <string.h>
 #ifdef HAVE_STDLIB_H
-#include <stdlib.h> /* for abort() as used in errors */
+#include <stdlib.h>
 #endif
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
@@ -124,7 +124,8 @@ librdf_serializer_register_factory(librdf_world *world,
 #endif
 
   if(!world->serializers) {
-    world->serializers=raptor_new_sequence((raptor_sequence_free_handler *)librdf_free_serializer_factory, NULL);
+    world->serializers = raptor_new_sequence((raptor_data_free_handler)librdf_free_serializer_factory, NULL);
+
     if(!world->serializers)
       goto oom;
   }
@@ -183,7 +184,7 @@ librdf_serializer_register_factory(librdf_world *world,
 /**
  * librdf_get_serializer_factory:
  * @world: redland world object
- * @name: the name of the factory (NULL or empty string if don't care)
+ * @name: the name of the factory (or NULL or empty string if don't care)
  * @mime_type: the MIME type of the syntax (NULL or empty string if not used)
  * @type_uri: URI of syntax (NULL if not used)
  *
@@ -267,6 +268,8 @@ librdf_get_serializer_factory(librdf_world *world,
  *
  * Get information on serializers.
  * 
+ * @Deprecated: use librdf_serializer_get_description() to return more information in a static structure.
+ *
  * Return value: non 0 on failure of if counter is out of range
  **/
 int
@@ -291,6 +294,26 @@ librdf_serializer_enumerate(librdf_world* world,
 }
 
 
+/**
+ * librdf_serializer_get_description:
+ * @world: world object
+ * @counter: index into the list of serializers
+ *
+ * Get serializer descriptive syntax information
+ * 
+ * Return value: description or NULL if counter is out of range
+ **/
+const raptor_syntax_description*
+librdf_serializer_get_description(librdf_world* world, 
+                                  unsigned int counter)
+{
+  librdf_world_open(world);
+
+  return raptor_world_get_serializer_description(world->raptor_world_ptr,
+                                                 counter);
+}
+
+
 /**
  * librdf_serializer_check_name:
  * @world: redland world object
@@ -324,7 +347,7 @@ librdf_serializer_check_name(librdf_world* world, const char *name)
 /**
  * librdf_new_serializer:
  * @world: redland world object
- * @name: the serializer factory name
+ * @name: the serializer factory name (or NULL or empty string if don't care)
  * @mime_type: the MIME type of the syntax (NULL if not used)
  * @type_uri: URI of syntax (NULL if not used)
  *
@@ -341,9 +364,23 @@ librdf_new_serializer(librdf_world *world,
 
   librdf_world_open(world);
 
-  factory=librdf_get_serializer_factory(world, name, mime_type, type_uri);
-  if(!factory)
+  factory = librdf_get_serializer_factory(world, name, mime_type, type_uri);
+  if(!factory) {
+    if(name)
+      librdf_log(world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER, NULL,
+                 "serializer '%s' not found", name);
+    else if(mime_type)
+      librdf_log(world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER, NULL,
+                 "serializer for mime_type '%s' not found", mime_type);
+    else if(type_uri)
+      librdf_log(world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER, NULL,
+                 "serializer for type URI '%s' not found", 
+                 librdf_uri_as_string(type_uri));
+    else
+      librdf_log(world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER, NULL,
+                 "default serializer not found");
     return NULL;
+  }
 
   return librdf_new_serializer_from_factory(world, factory);
 }
@@ -402,8 +439,9 @@ librdf_new_serializer_from_factory(librdf_world *world,
 void
 librdf_free_serializer(librdf_serializer *serializer) 
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(serializer, librdf_serializer);
-
+  if(!serializer)
+    return;
+  
   if(serializer->context) {
     if(serializer->factory->terminate)
       serializer->factory->terminate(serializer->context);
@@ -689,6 +727,7 @@ librdf_serializer_serialize_model_to_string(librdf_serializer* serializer,
  * @iostr: the #raptor_iostream to write to
  *
  * Write a #librdf_stream to a #raptor_iostream.
+ * This function takes ownership of the iostream and frees it.
  * 
  * Return value: non-0 on failure
  **/
@@ -716,6 +755,7 @@ librdf_serializer_serialize_stream_to_iostream(librdf_serializer* serializer,
  * @iostr: the #raptor_iostream to write to
  *
  * Write a serialized #librdf_model to a #raptor_iostream.
+ * This function takes ownership of the iostream and frees it.
  * 
  * Return value: non-0 on failure
  **/
