@@ -643,10 +643,13 @@ librdf_statement_encode_parts_internal(librdf_statement* statement,
     node_len = librdf_node_encode(context_node, p, length);
     if(!node_len)
       return 0;
+    /* p and length changes never needed to be calculated [clang] */
+    /*
     if(p) {
       p += node_len;
       length -= node_len;
     }
+    */
 
     total_length += node_len;
   }
@@ -714,6 +717,31 @@ librdf_statement_decode(librdf_statement* statement,
 }
 
 
+/**
+ * librdf_statement_decode_parts:
+ * @statement: the statement to deserialise into
+ * @context_node: pointer to #librdf_node context_node to deserialise into
+ * @buffer: the buffer to use
+ * @length: buffer size
+ *
+ * Decodes a statement + context node from a buffer. (ALWAYS FAILS)
+ * 
+ * Used to decode the serialised statement as created by
+ * librdf_statement_encode() from the given buffer.
+ *
+ * @Deprecated: Replaced by librdf_statement_decode2() which works.
+ * 
+ * Return value: 0 signifying failure
+ **/
+size_t
+librdf_statement_decode_parts(librdf_statement* statement, 
+                              librdf_node** context_node,
+                              unsigned char *buffer, size_t length)
+{
+  return 0;
+}
+
+
 /**
  * librdf_statement_decode2:
  * @world: redland world
@@ -884,6 +912,7 @@ main(int argc, char *argv[])
   librdf_free_statement(statement2);
   librdf_free_statement(statement);
 
+  raptor_free_iostream(iostr);
 
   librdf_free_world(world);
   
