@@ -456,43 +456,70 @@ librdf_storage_hashes_terminate(librdf_storage* storage)
 static int
 librdf_storage_hashes_clone(librdf_storage* new_storage, librdf_storage* old_storage)
 {
-  librdf_storage_hashes_instance* old_context=(librdf_storage_hashes_instance*)old_storage->instance;
-  
-  librdf_storage_hashes_instance* new_context=(librdf_storage_hashes_instance*)LIBRDF_CALLOC(
-    librdf_storage_hashes_instance, 1, sizeof(librdf_storage_hashes_instance));
-  
-  librdf_storage_set_instance(new_storage, new_context);
-  
+  librdf_storage_hashes_instance* old_context;
+  librdf_hash* new_options = NULL;
+  char* new_name = NULL;
+  char* new_hash_type = NULL;
+  char* new_db_dir = NULL;
+  char* new_indexes = NULL;
+
+  old_context = (librdf_storage_hashes_instance*)old_storage->instance;
+
   /* Bump up old context name if any */
-  if(old_context->name) {  
-    new_context->name=librdf_heuristic_gen_name(old_context->name);
-    if(!new_context->name)
-      return 1;
+  if(old_context->name) {
+    new_name = librdf_heuristic_gen_name(old_context->name);
+    if(!new_name)
+      goto failed;
   }
   
   /* This is always a copy of an in-memory hash */
-  new_context->options=librdf_new_hash_from_hash(old_context->options);
-  if(!new_context->options) {
-    if(new_context->name)
-      LIBRDF_FREE(cstring, new_context->name);
-    return 1;
+  new_options = librdf_new_hash_from_hash(old_context->options);
+  if(!new_options)
+    goto failed;
+
+  if(old_context->hash_type) {
+    new_hash_type =  strdup(old_context->hash_type);
+    if(!new_hash_type)
+      goto failed;
   }
 
-  if(librdf_storage_hashes_init_common(new_storage, new_context->name,
-                                       old_context->hash_type,
-                                       old_context->db_dir,
-                                       old_context->indexes,
+  if(old_context->db_dir) {
+    new_db_dir = strdup(old_context->db_dir);
+    if(!new_db_dir)
+      goto failed;
+  }
+
+  if(old_context->indexes) {
+    new_indexes = strdup(old_context->indexes);
+    if(!new_indexes)
+      goto failed;
+  }
+
+  if(librdf_storage_hashes_init_common(new_storage, new_name,
+                                       new_hash_type,
+                                       new_db_dir,
+                                       new_indexes,
                                        old_context->mode,
                                        old_context->is_writable,
                                        old_context->is_new,
-                                       new_context->options)) {
-    librdf_free_hash(new_context->options);
-    if(new_context->name)
-      LIBRDF_FREE(cstring, new_context->name);
-    return 1;
+                                       new_options)) {
+    goto failed;
   }
 
   return 0;
+
+  failed:
+  if(new_name)
+    LIBRDF_FREE(cstring, new_name);
+  if(new_hash_type)
+    LIBRDF_FREE(cstring, new_hash_type);
+  if(new_db_dir)
+    LIBRDF_FREE(cstring, new_db_dir);
+  if(new_indexes)
+    LIBRDF_FREE(cstring, new_indexes);
+  if(new_options)
+    librdf_free_hash(new_options);
+  return 1;
 }
  
 
